@@ -1,11 +1,22 @@
-.ggemmeans_zi_predictions <- function(model, model_frame, preds, ci.lvl, terms, cleaned_terms, value_adjustment, condition, nsim = 1000, type = "fe") {
+.ggemmeans_zi_predictions <- function(model,
+                                      model_frame,
+                                      preds,
+                                      ci_level,
+                                      interval,
+                                      terms,
+                                      cleaned_terms,
+                                      typical,
+                                      condition,
+                                      nsim = 1000,
+                                      type = "fixed") {
   prdat <- exp(preds$x1$emmean) * (1 - stats::plogis(preds$x2$emmean))
 
   # compute ci, two-ways
-  if (!is.null(ci.lvl) && !is.na(ci.lvl))
-    ci <- (1 + ci.lvl) / 2
-  else
+  if (!is.null(ci_level) && !is.na(ci_level)) {
+    ci <- (1 + ci_level) / 2
+  } else {
     ci <- 0.975
+  }
 
   # degrees of freedom
   dof <- .get_df(model)
@@ -16,7 +27,7 @@
     model = model,
     model_frame = model_frame,
     terms = terms,
-    value_adjustment = value_adjustment,
+    typical = typical,
     factor_adjustment = FALSE,
     show_pretty_message = FALSE,
     condition = condition
@@ -27,25 +38,27 @@
     model = model,
     model_frame = model_frame,
     terms = terms,
-    value_adjustment = value_adjustment,
+    typical = typical,
     show_pretty_message = FALSE,
     condition = condition,
-    emmeans.only = FALSE,
+    emmeans_only = FALSE,
     verbose = FALSE
   )
 
   # Since the zero inflation and the conditional model are working in "opposite
   # directions", confidence intervals can not be derived directly  from the
-  # "predict()"-function. Thus, confidence intervals for type = "fe.zi" are
+  # "predict()"-function. Thus, confidence intervals for type = "zero_inflated" are
   # based on quantiles of simulated draws from a multivariate normal distribution
   # (see also _Brooks et al. 2017, pp.391-392_ for details).
 
-  prdat.sim <- .simulate_zi_predictions(model, newdata, nsim, terms, value_adjustment, condition)
+  prdat.sim <- .simulate_zi_predictions(model, newdata, nsim, terms, typical, condition)
 
-  if (is.null(prdat.sim)) {
-    insight::format_error(
-      "Predicted values could not be computed. Try reducing number of simulation, using argument `nsim` (e.g. `nsim = 100`)"
-    )
+  if (is.null(prdat.sim) || inherits(prdat.sim, c("simpleError", "simpleWarning")) || !length(prdat.sim)) {
+    msg <- "Predicted values could not be computed. Try reducing number of simulation, using argument `nsim` (e.g. `nsim = 100`)." # nolint
+    if (inherits(prdat.sim, c("simpleError", "simpleWarning"))) {
+      msg <- c(msg, "\nFurthermore, following error occured:", prdat.sim$message)
+    }
+    insight::format_error(msg)
   }
 
   # we need two data grids here: one for all combination of levels from the
@@ -59,7 +72,7 @@
   sims <- exp(prdat.sim$cond) * (1 - stats::plogis(prdat.sim$zi))
   prediction_data <- .join_simulations(data_grid, newdata, prdat, sims, ci, cleaned_terms)
 
-  if (type == "re.zi") {
+  if (identical(interval, "prediction")) {
     revar <- .get_residual_variance(model)
     # get link-function and back-transform fitted values
     # to original scale, so we compute proper CI
