@@ -1,17 +1,18 @@
 # "factor_adjustment" indicates if factors should be held constant or not
 # need to be false for computing std.error for merMod objects
 
-# value_adjustment is the function to calculate at which value non-focal
+# typical is the function to calculate at which value non-focal
 # terms are held constant (mean, median, ...)
 
 .data_grid <- function(model,
                        model_frame,
                        terms,
-                       value_adjustment,
+                       typical,
                        factor_adjustment = TRUE,
                        show_pretty_message = TRUE,
                        condition = NULL,
-                       emmeans.only = FALSE,
+                       emmeans_only = FALSE,
+                       include_random = FALSE,
                        verbose = TRUE) {
   # special handling for coxph
   if (inherits(model, c("coxph", "coxme"))) {
@@ -28,35 +29,57 @@
   # however, numeric in the original data. We need to coerce to factor here,
   # and back to numeric later...
   on_the_fly_factors <- attributes(model_frame)$factors
-  model_frame[] <- lapply(model_frame, function(i) if (isTRUE(attributes(i)$factor)) as.factor(i) else i)
+  model_frame[] <- lapply(model_frame, function(i) {
+    if (isTRUE(attributes(i)$factor)) as.factor(i) else i
+  })
 
-  # check for logical variables, might not work
-  if (any(vapply(model_frame, is.logical, logical(1)))) {
-    insight::format_error(
-      "Variables of type `logical` do not work, please coerce to factor and fit the model again."
-    )
+  # special handling for fixest, when "cluster" is no factor
+  if (inherits(model, "fixest")) {
+    fixest_cluster <- insight::find_variables(model)$cluster
+    if (!is.null(fixest_cluster)) {
+      for (fc in fixest_cluster) {
+        if (!is.factor(model_frame[[fc]])) {
+          model_frame[[fc]] <- as.factor(model_frame[[fc]])
+        }
+      }
+    }
   }
 
-  # any weights?
-  w <- insight::get_weights(model)
-  if (is.null(w) || all(w == 1)) w <- NULL
+  # check for logical variables, might not work
+  logicals <- vapply(model_frame, is.logical, logical(1))
+  if (any(logicals)) {
+    insight::format_error(
+      "Variables of type `logical` do not work, please coerce to factor and fit the model again.",
+      paste0(
+        "The following variables in are of type logical: ",
+        toString(paste0("`", colnames(model_frame)[logicals], "`"))
+      )
+    )
+  }
 
   # get random effects (grouping factor)
   random_effect_terms <- insight::find_random(model, split_nested = TRUE, flatten = TRUE)
 
-  ## TODO check for other panelr models
-
   # clean variable names
-  # if (!inherits(model, "wbm")) {
   colnames(model_frame) <- insight::clean_names(colnames(model_frame))
-  # }
 
-
-  # get specific levels
+  # get specific levels. "focal_terms" is now a names list, with the variable
+  # names as names, and the specified values (in []) as list elements.
   focal_terms <- .get_representative_values(terms, model_frame)
   # and all specified variables
   all_terms <- .clean_terms(terms)
 
+  # check for character variables, might not work
+  characters <- vapply(model_frame[all_terms], is.character, logical(1))
+  if (any(characters) && verbose) {
+    insight::format_alert(
+      "Some of the focal terms are of type `character`. This may lead to unexpected results. It is recommended to convert these variables to factors before fitting the model.", # nolint
+      paste0(
+        "The following variables are of type character: ",
+        toString(paste0("`", all_terms[characters], "`"))
+      )
+    )
+  }
 
   # check if user has any predictors with log-transformation inside
   # model formula, but *not* used back-transformation "exp". Tell user
@@ -69,16 +92,24 @@
         any_offset_log_term <- .get_offset_log_terms(model)
         # check if we have offset() in formula, with transformed variable
         if (any(any_offset_log_term)) {
-          clean.term <- insight::find_predictors(model, effects = "all", component = "all", flatten = FALSE)
+          clean.term <- insight::find_predictors(
+            model,
+            effects = "all",
+            component = "all",
+            flatten = FALSE,
+            verbose = FALSE
+          )
           clean.term <- unlist(clean.term[c("conditional", "random", "instruments")])[any_offset_log_term]
 
           # try to back-transform
           offset_function <- .get_offset_transformation(model)
-          if (identical(offset_function, "log")) {
+          # check whether offset is transformed and *not* fixed (conditioned) at a specific value
+          if (identical(offset_function, "log") && !clean.term %in% names(condition)) {
             if (verbose) {
-              insight::format_warning(
+              insight::format_alert(
                 "Model uses a transformed offset term. Predictions may not be correct.",
-                sprintf("Please apply transformation of offset term to the data before fitting the model and use `offset(%s)` in the model formula.", clean.term)
+                sprintf("It is recommended to fix the offset term using the `condition` argument, e.g. `condition = c(%s = 1)`.", clean.term), # nolint
+                sprintf("You could also transform the offset variable before fitting the model and use `offset(%s)` in the model formula.", clean.term) # nolint
               )
             }
             olt <- clean.term
@@ -103,44 +134,38 @@
   use_all_values <- FALSE
 
   # for these models, always all values are used
-  all_values_models <- c("Gam", "gam", "vgam", "glm", "lm", "brmsfit", "bamlss", "gamlss", "glmx", "feglm")
+  all_values_models <- c("Gam", "gam", "vgam", "glm", "lm", "nestedLogit",
+                         "brmsfit", "bamlss", "gamlss", "glmx", "feglm")
 
-  if (.has_splines(model) && !.uses_all_tag(terms)) {
-    if (inherits(model, all_values_models)) {
-      use_all_values <- TRUE
-    } else if (show_pretty_message && verbose) {
+  # for these models, all values are used if we have splines or polynomial terms
+  if (.has_spline_or_poly(model) && !.uses_all_tag(terms)) {
+    use_all_values <- inherits(model, all_values_models)
+  }
+
+  # for remaining models, if not "[all]" tag is used, tell user about
+  # considerung using the "[all]" tag
+  if (verbose && !use_all_values && !.uses_all_tag(terms)) {
+    if (.has_splines(model)) {
       insight::format_alert(sprintf(
-        "Model contains splines or polynomial terms. Consider using `terms=\"%s [all]\"` to get smooth plots. See also package-vignette 'Marginal Effects at Specific Values'.", all_terms[1]
+        "Model contains splines or polynomial terms. Consider using `terms=\"%s [all]\"` to get smooth plots. See also package-vignette 'Adjusted Predictions at Specific Values'.", all_terms[1] # nolint
+      ))
+      show_pretty_message <- FALSE
+    }
+    if (.has_poly(model) && show_pretty_message) {
+      insight::format_alert(sprintf(
+        "Model contains polynomial or cubic / quadratic terms. Consider using `terms=\"%s [all]\"` to get smooth plots. See also package-vignette 'Adjusted Predictions at Specific Values'.", all_terms[1] # nolint
+      ))
+      show_pretty_message <- FALSE
+    }
+    if (.has_trigonometry(model) && show_pretty_message) {
+      insight::format_alert(sprintf(
+        "Model contains trigonometric terms (sinus, cosinus, ...). Consider using `terms=\"%s [all]\"` to get smooth plots. See also package-vignette 'Adjusted Predictions at Specific Values'.", all_terms[1] # nolint
       ))
       show_pretty_message <- FALSE
     }
   }
 
-  if (.has_poly(model) && !.uses_all_tag(terms) && !use_all_values) {
-    if (inherits(model, all_values_models)) {
-      use_all_values <- TRUE
-    } else if (show_pretty_message && verbose) {
-      insight::format_alert(sprintf(
-        "Model contains polynomial or cubic / quadratic terms. Consider using `terms=\"%s [all]\"` to get smooth plots. See also package-vignette 'Marginal Effects at Specific Values'.", all_terms[1]
-      ))
-      show_pretty_message <- FALSE
-    }
-  }
-
-
-  if (.has_trigonometry(model) && !.uses_all_tag(terms) && !use_all_values) {
-    if (inherits(model, all_values_models)) {
-      use_all_values <- TRUE
-    } else if (show_pretty_message && verbose) {
-      insight::format_alert(sprintf(
-        "Model contains trigonometric terms (sinus, cosinus, ...). Consider using `terms=\"%s [all]\"` to get smooth plots. See also package-vignette 'Marginal Effects at Specific Values'.", all_terms[1]
-      ))
-      show_pretty_message <- FALSE
-    }
-  }
-
-
-  # find terms for which no specific values are given
+  # find terms for which no specific at-values are given
   conditional_terms <- which(!(all_terms %in% names(focal_terms)))
 
   # prettify numeric vectors, get representative values
@@ -149,7 +174,7 @@
     original_model_frame = model_frame,
     terms = all_terms,
     use_all_values = use_all_values,
-    show_pretty_message = show_pretty_message && model_info$is_binomial
+    show_pretty_message = show_pretty_message && model_info$is_binomial && verbose
   )
   names(constant_levels) <- all_terms[conditional_terms]
   focal_terms <- c(focal_terms, constant_levels)
@@ -161,10 +186,11 @@
   # in the data grid that actually appear in the data
 
   if (inherits(model, "brmsfit")) {
-    model_terms <- insight::find_terms(model, flatten = TRUE)
+    model_terms <- insight::find_terms(model, flatten = TRUE, verbose = FALSE)
     monotonics <- grepl("mo\\((.*)\\)", model_terms)
     if (any(monotonics)) {
       mo_terms <- gsub("mo\\((.*)\\)", "\\1", model_terms[monotonics])
+      # now check if values are valid
       invalid_levels <- unlist(lapply(mo_terms, function(mt) {
         if (mt %in% names(focal_terms) && mt %in% colnames(model_frame)) {
           !all(model_frame[[mt]] %in% focal_terms[[mt]])
@@ -174,26 +200,31 @@
       }))
       if (any(invalid_levels)) {
         insight::format_error(
-          sprintf("Variable(s) %s are used as monotonic effects, however, only values that are also present in the data are allowed for predictions.", toString(mo_terms)),
+          sprintf("Variable(s) %s are used as monotonic effects, however, only values that are also present in the data are allowed for predictions.", toString(mo_terms)), # nolint
           "Consider converting variables used in `mo()` into (ordered) factors before fitting the model."
         )
       }
+      # we also need to remove monotonic terms from "on-the-fly" factors,
+      # because we need the levels from the monotonic term
+      on_the_fly_factors <- setdiff(on_the_fly_factors, mo_terms)
+      if (!length(on_the_fly_factors)) {
+        on_the_fly_factors <- NULL
+      }
     }
   }
 
 
   ## TODO check for other panelr models
 
-  # get names of all predictor variable
-  # if (inherits(model, "wbm")) {
-  #   model_predictors <- colnames(model_frame)
-  # } else {
-  #   model_predictors <- insight::find_predictors(model, effects = "all", component = "all", flatten = TRUE)
-  # }
-
-  offset_term <- .offset_term(model, show_pretty_message)
+  offset_term <- .offset_term(model, condition, show_pretty_message)
   model_predictors <- c(
-    insight::find_predictors(model, effects = "all", component = "all", flatten = TRUE),
+    insight::find_predictors(
+      model,
+      effects = "all",
+      component = "all",
+      flatten = TRUE,
+      verbose = FALSE
+    ),
     offset_term
   )
   if (inherits(model, "wbm")) {
@@ -217,27 +248,25 @@
   # remove NA from values, so we don't have expanded data grid
   # with missing values. this causes an error with predict()
 
-  if (any(sapply(focal_terms, anyNA))) {
+  if (any(vapply(focal_terms, anyNA, TRUE))) {
     focal_terms <- lapply(focal_terms, function(.x) as.vector(stats::na.omit(.x)))
   }
 
 
-  ## TODO check, it should actually no longer happen that
-  # the values of "model_predictors" are not in the column names of
-  # the model frame "model_frame"
+  ## TODO check, it should actually no longer happen that the values of
+  # "model_predictors" are not in the column names of the model frame "model_frame"
 
-  # names of predictor variables may vary, e.g. if log(x)
-  # or poly(x) etc. is used. so check if we have correct
-  # predictor names that also appear in model frame
+  # names of predictor variables may vary, e.g. if log(x) or poly(x) etc. is used.
+  # so check if we have correct predictor names that also appear in model frame
 
   ## TODO brms does currently not support "terms()" generic
 
-  if (!inherits(model, "wbm")) {
-
+  if (inherits(model, "wbm")) {
+    model_predictors <- model_predictors[model_predictors %in% colnames(model_frame)]
+  } else {
     if (sum(!(model_predictors %in% colnames(model_frame))) > 0 && !inherits(model, c("brmsfit", "MCMCglmm"))) {
       # get terms from model directly
-      model_predictors <- tryCatch(attr(stats::terms(model), "term.labels", exact = TRUE),
-                                        error = function(e) NULL)
+      model_predictors <- .safe(attr(stats::terms(model), "term.labels", exact = TRUE))
     }
 
     # 2nd check
@@ -245,30 +274,62 @@
       # get terms from model frame column names
       model_predictors <- colnames(model_frame)
       # we may have more terms now, e.g. intercept. remove those now
-      if (length(model_predictors) > n_predictors) model_predictors <- model_predictors[2:(n_predictors + 1)]
+      if (length(model_predictors) > n_predictors) {
+        model_predictors <- model_predictors[2:(n_predictors + 1)]
+      }
     }
-
-  } else {
-    model_predictors <- model_predictors[model_predictors %in% colnames(model_frame)]
   }
 
   # keep those, which we did not process yet
   model_predictors <- model_predictors[!(model_predictors %in% names(focal_terms))]
 
+  # any weights?
+  w <- insight::get_weights(model)
+  if (is.null(w) || all(w == 1)) {
+    w <- NULL
+  }
+
   # if we have weights, and typical value is mean, use weighted means
   # as function for the typical values
 
-  if (!.is_empty(w) && length(w) == nrow(model_frame) && value_adjustment == "mean") {
-    value_adjustment <- "weighted.mean"
+  if (!.is_empty(w) && length(w) == nrow(model_frame)) {
+    if (identical(typical, "mean")) {
+      typical <- "weighted.mean"
+    } else if (is.list(typical)) {
+      typical <- lapply(typical, function(x) {
+        if (identical(x, "mean")) {
+          "weighted.mean"
+        } else {
+          x
+        }
+      })
+    }
   }
 
-  if (value_adjustment == "weighted.mean" && .is_empty(w)) {
-    value_adjustment <- "mean"
+  if (.is_empty(w)) {
+    if (identical(typical, "weighted.mean")) {
+      typical <- "mean"
+    } else if (is.list(typical)) {
+      typical <- lapply(typical, function(x) {
+        if (identical(x, "weighted.mean")) {
+          "mean"
+        } else {
+          x
+        }
+      })
+    }
+  }
+
+  # at this point, we have the "terms" in focal terms, but possibly in different
+  # order as in "terms" (because we first have terms with [] in focal terms, then
+  # the remaining). Here we make sure we restore the correct order
+
+  if (all(all_terms %in% names(focal_terms))) {
+    focal_terms <- focal_terms[all_terms]
   }
 
 
-  # do we have variables that should be held constant at a
-  # specific value?
+  # do we have variables that should be held constant at a specific value?
 
   if (!is.null(condition) && !is.null(names(condition))) {
     focal_terms <- c(focal_terms, as.list(condition))
@@ -281,7 +342,7 @@
   # reference level for factors and most common element for character vectors
 
   # special handling for emmeans
-  if (isTRUE(emmeans.only)) {
+  if (isTRUE(emmeans_only)) {
     # check for log-terms, and if in focal terms, remove "0" from values
     log_terms <- .which_log_terms(model)
     if (!is.null(log_terms) && any(log_terms %in% names(focal_terms))) {
@@ -295,8 +356,13 @@
     constant_values <- lapply(model_predictors, function(x) {
       pred <- model_frame[[x]]
       if (!is.factor(pred) && !is.character(pred) && !x %in% random_effect_terms) {
-        .typical_value(pred, fun = value_adjustment, weights = w, predictor = x,
-                       log_terms = .which_log_terms(model), emmeans.only = emmeans.only)
+        .typical_value(
+          pred,
+          fun = typical,
+          weights = w,
+          predictor = x,
+          log_terms = .which_log_terms(model)
+        )
       }
     })
     names(constant_values) <- model_predictors
@@ -306,9 +372,16 @@
     # adjust constant values, factors set to reference level
     constant_values <- lapply(model_predictors, function(x) {
       pred <- model_frame[[x]]
-      if (is.factor(pred)) pred <- droplevels(pred)
-      .typical_value(pred, fun = value_adjustment, weights = w, predictor = x,
-                     log_terms = .which_log_terms(model))
+      if (is.factor(pred)) {
+        pred <- droplevels(pred)
+      }
+      .typical_value(
+        pred,
+        fun = typical,
+        weights = w,
+        predictor = x,
+        log_terms = .which_log_terms(model)
+      )
     })
     names(constant_values) <- model_predictors
 
@@ -327,7 +400,7 @@
         levels(droplevels(x))
       } else {
         if (is.factor(x)) x <- droplevels(x)
-        .typical_value(x, fun = value_adjustment, weights = w)
+        .typical_value(x, fun = typical, weights = w)
       }
     })
     names(constant_values) <- model_predictors
@@ -357,7 +430,7 @@
 
   # for MixMod, we need mean value of response as well...
   if (inherits(model, c("MixMod", "MCMCglmm"))) {
-    constant_values <- c(constant_values, .typical_value(insight::get_response(model)))
+    constant_values <- c(constant_values, .typical_value(insight::get_response(model, verbose = FALSE)))
     names(constant_values)[length(constant_values)] <- insight::find_response(model, combine = FALSE)
   }
 
@@ -367,7 +440,7 @@
 
   # stop here for emmeans-objects
 
-  if (isTRUE(emmeans.only)) {
+  if (isTRUE(emmeans_only)) {
 
     # remove grouping factor of RE from constant values
     # only applicable for MixMod objects
@@ -382,19 +455,22 @@
     # restore original type
     focal_terms <- lapply(focal_term_names, function(x) {
       # check for consistent vector type: on-the-fly conversion of factors
-      if (!is.null(on_the_fly_factors) && x %in% on_the_fly_factors)
+      if (!is.null(on_the_fly_factors) && x %in% on_the_fly_factors) {
         return(.factor_to_numeric(focal_terms[[x]]))
+      }
 
       # check for consistent vector type: numeric
-      if (is.numeric(model_frame[[x]]) && !is.numeric(focal_terms[[x]]))
+      if (is.numeric(model_frame[[x]]) && !is.numeric(focal_terms[[x]])) {
         return(.factor_to_numeric(focal_terms[[x]]))
+      }
 
       # check for consistent vector type: factor
-      if (is.factor(model_frame[[x]]) && !is.factor(focal_terms[[x]]))
+      if (is.factor(model_frame[[x]]) && !is.factor(focal_terms[[x]])) {
         return(as.character(focal_terms[[x]]))
+      }
 
       # else return original vector
-      return(focal_terms[[x]])
+      focal_terms[[x]]
     })
 
     # add back names
@@ -430,17 +506,25 @@
   datlist <- lapply(colnames(dat), function(x) {
 
     # check for consistent vector type: numeric
-    if (is.numeric(model_frame[[x]]) && !is.numeric(dat[[x]]))
+    if (is.numeric(model_frame[[x]]) && !is.numeric(dat[[x]])) {
       return(.factor_to_numeric(dat[[x]]))
+    }
 
     # check for consistent vector type: factor
-    if (is.factor(model_frame[[x]]) && !is.factor(dat[[x]]))
+    if (is.factor(model_frame[[x]]) && !is.factor(dat[[x]])) {
       return(as.factor(dat[[x]]))
+    }
 
     # else return original vector
-    return(dat[[x]])
+    dat[[x]]
   })
 
+  # if we want to average across random effects, we set include_random to TRUE
+  # in this case, we create a data grid for all levels of the random effects
+  # and average across groups later...
+  if (include_random) {
+    focal_terms <- c(focal_terms, random_effect_terms)
+  }
 
   # get list names. we need to remove patterns like "log()" etc.
   names(datlist) <- names(focal_terms)
@@ -461,11 +545,14 @@
   # which will return predictions on a population level.
   # See ?glmmTMB::predict
 
-  if (inherits(model, c("glmmTMB", "merMod", "rlmerMod", "MixMod", "brmsfit", "lme"))) {
+  if (!include_random && inherits(model, c("glmmTMB", "merMod", "rlmerMod", "MixMod", "brmsfit", "lme"))) {
     cleaned_terms <- .clean_terms(terms)
 
     # check if we have fixed effects as grouping factor in random effects as well...
-    cleaned_terms <- unique(c(cleaned_terms, insight::find_predictors(model, effects = "fixed", flatten = TRUE)))
+    cleaned_terms <- unique(c(
+      cleaned_terms,
+      insight::find_predictors(model, effects = "fixed", flatten = TRUE, verbose = FALSE)
+    ))
     # if so, remove from random-effects here
     random_effect_terms <- random_effect_terms[!(random_effect_terms %in% cleaned_terms)]
 
@@ -528,8 +615,14 @@
 
   w <- insight::find_weights(model)
   if (!is.null(w) && !inherits(model, "brmsfit")) {
-    datlist$.w <- NA_real_
-    colnames(datlist)[ncol(datlist)] <- w
+    # don't overwrite variables
+    w <- setdiff(w, colnames(datlist))
+    # for lme, can't be NA
+    if (inherits(model, c("lme", "gls"))) {
+      datlist[w] <- 1
+    } else {
+      datlist[w] <- NA_real_
+    }
   }
 
   datlist
@@ -539,14 +632,13 @@
 
 .add_offset_to_mf <- function(x, model_frame, offset_term) {
   # first try, parent frame
-  dat <- tryCatch(eval(x$call$data, envir = parent.frame()), error = function(e) NULL)
+  dat <- .safe(eval(x$call$data, envir = parent.frame()))
 
   if (is.null(dat)) {
     # second try, global env
-    dat <- tryCatch(eval(x$call$data, envir = globalenv()), error = function(e) NULL)
+    dat <- .safe(eval(x$call$data, envir = globalenv()))
   }
 
-
   if (!is.null(dat) && .obj_has_name(x$call, "subset")) {
     dat <- subset(dat, subset = eval(x$call$subset))
   }
@@ -561,3 +653,21 @@
     }
   )
 }
+
+
+.get_model_data <- function(model) {
+  # get model frame
+  model_frame <- insight::get_data(model, source = "frame", verbose = FALSE)
+
+  # sanity check - could data be extracted from model frame?
+  if (is.null(model_frame)) {
+    model_frame <- .safe(insight::get_data(model, source = "environment", verbose = FALSE))
+  }
+
+  # tibbles are not supported
+  if (inherits(model_frame, c("tbl", "tbl_df"))) {
+    class(model_frame) <- "data.frame"
+  }
+
+  model_frame
+}
